@@ -1,20 +1,29 @@
 import os
 import os.path as op
+import shutil
 import sys
+import tempfile
 import textwrap
 import zipfile2
 
 import mock
 
-if sys.version_info[:2] < (2, 7):
+if sys.version_info < (2, 7):  # noqa
     import unittest2 as unittest
 else:
     import unittest
 
-from ...errors import InvalidEggName, InvalidMetadata, UnsupportedMetadata
-from ...platforms import EPDPlatform
+from ...errors import (
+    InvalidEggName, InvalidMetadataField, MissingMetadata, UnsupportedMetadata,
+    InvalidRequirementStringHyphen
+)
+from ...utils import tempdir
+from ...utils.test_data import (
+    MKL_10_3_RH5_X86_64, NOSE_1_3_4_RH6_X86_64, NOSE_1_3_4_RH5_X86_64
+)
+from ...platforms import EPDPlatform, PlatformABI
 from ...platforms.legacy import LegacyEPDPlatform
-from ...versions import EnpkgVersion
+from ...versions import EnpkgVersion, MetadataVersion
 
 from .._egg_info import (
     Requirement, EggMetadata, LegacySpecDepend, parse_rawspec,
@@ -22,74 +31,85 @@ from .._egg_info import (
 )
 
 from .common import (
-    BROKEN_MCCABE_EGG, DATA_DIR, ENSTALLER_EGG, ETS_EGG, FAKE_PYSIDE_1_1_0_EGG,
-    FAKE_PYSIDE_1_1_0_EGG_PKG_INFO, MKL_EGG, NUMEXPR_2_2_2_EGG,
-    PYMULTINEST_EGG, _OSX64APP_EGG, PYSIDE_1_0_3_EGG, XZ_5_2_0_EGG
+    BROKEN_MCCABE_EGG, DATA_DIR, ENSTALLER_EGG, ETS_EGG,
+    FAKE_MEDIALOG_BOARDFILE_1_6_1_EGG, FAKE_MEDIALOG_BOARDFILE_1_6_1_PKG_INFO,
+    MKL_EGG, NUMEXPR_2_2_2_EGG, PYMULTINEST_EGG, _OSX64APP_EGG,
+    PYSIDE_1_0_3_EGG, XZ_5_2_0_EGG
 )
 
 
+M = MetadataVersion.from_string
+
+
 class TestRequirement(unittest.TestCase):
     def test_str(self):
-        dependency = Requirement(name="numpy")
+        dependency = Requirement(name=u"numpy")
         r_str = "numpy"
 
         self.assertEqual(r_str, str(dependency))
 
-        dependency = Requirement(name="numpy", version_string="1.7.1")
+        dependency = Requirement(name=u"numpy", version_string=u"1.7.1")
         r_str = "numpy 1.7.1"
 
         self.assertEqual(r_str, str(dependency))
 
-        dependency = Requirement(name="numpy", version_string="1.7.1",
+        dependency = Requirement(name=u"numpy", version_string=u"1.7.1",
                                  build_number=1)
         r_str = "numpy 1.7.1-1"
 
         self.assertEqual(r_str, str(dependency))
 
-        dependency = Requirement("numpy", "1.7.1", 1)
+        dependency = Requirement(u"numpy", u"1.7.1", 1)
         r_str = "numpy 1.7.1-1"
 
         self.assertEqual(r_str, str(dependency))
 
     def test_from_spec_string(self):
-        dependency = Requirement.from_spec_string("numpy")
+        dependency = Requirement.from_spec_string(u"numpy")
         self.assertEqual(dependency.name, "numpy")
         self.assertEqual(dependency.version_string, "")
         self.assertEqual(dependency.build_number, -1)
         self.assertEqual(dependency.strictness, 1)
 
-        dependency = Requirement.from_spec_string("numpy 1.7.1")
+        dependency = Requirement.from_spec_string(u"numpy 1.7.1")
         self.assertEqual(dependency.name, "numpy")
         self.assertEqual(dependency.version_string, "1.7.1")
         self.assertEqual(dependency.build_number, -1)
         self.assertEqual(dependency.strictness, 2)
 
-        dependency = Requirement.from_spec_string("numpy 1.7.1-2")
+        dependency = Requirement.from_spec_string(u"numpy 1.7.1-2")
         self.assertEqual(dependency.name, "numpy")
         self.assertEqual(dependency.version_string, "1.7.1")
         self.assertEqual(dependency.build_number, 2)
         self.assertEqual(dependency.strictness, 3)
 
+        with self.assertRaisesRegexp(
+                InvalidRequirementStringHyphen,
+                "Invalid requirement string {0!r}:"
+                " Package versions should be separated by whitespace"
+                " instead of a hyphen.".format(u"numpy-1.7.1-2")):
+            Requirement.from_spec_string(u"numpy-1.7.1-2")
+
     def test_from_string(self):
-        dependency = Requirement.from_string("numpy-1.7.1-2", 3)
+        dependency = Requirement.from_string(u"numpy-1.7.1-2", 3)
         self.assertEqual(dependency.name, "numpy")
         self.assertEqual(dependency.version_string, "1.7.1")
         self.assertEqual(dependency.build_number, 2)
 
-        dependency = Requirement.from_string("numpy-1.7.1-2", 2)
+        dependency = Requirement.from_string(u"numpy-1.7.1-2", 2)
         self.assertEqual(dependency.name, "numpy")
         self.assertEqual(dependency.version_string, "1.7.1")
         self.assertEqual(dependency.build_number, -1)
 
-        dependency = Requirement.from_string("numpy-1.7.1-2", 1)
+        dependency = Requirement.from_string(u"numpy-1.7.1-2", 1)
         self.assertEqual(dependency.name, "numpy")
         self.assertEqual(dependency.version_string, "")
         self.assertEqual(dependency.build_number, -1)
 
         self.assertRaises(InvalidEggName, lambda:
-                          Requirement.from_string("numpy"))
+                          Requirement.from_string(u"numpy"))
         self.assertRaises(InvalidEggName, lambda:
-                          Requirement.from_string("numpy 1.7.1"))
+                          Requirement.from_string(u"numpy 1.7.1"))
 
 
 class TestLegacySpecDepend(unittest.TestCase):
@@ -133,7 +153,7 @@ packages = [
         # Then
         self.assertMultiLineEqual(depend.to_string(), r_depend)
         self.assertEqual(depend.packages,
-                         [Requirement.from_spec_string("Qt 4.8.5")])
+                         [Requirement.from_spec_string(u"Qt 4.8.5")])
 
     def test_to_string(self):
         # Given
@@ -183,6 +203,7 @@ packages = [
             depend._epd_legacy_platform,
             LegacyEPDPlatform.from_epd_platform_string("win-32")
         )
+        self.assertEqual(depend.platform_abi, "msvc2008")
 
     def test_format_1_3(self):
         r_depend = """\
@@ -206,37 +227,98 @@ packages = [
         depend = LegacySpecDepend.from_string(r_depend)
 
         # Then
-        self.assertEqual(depend._metadata_version, "1.3")
+        self.assertEqual(depend._metadata_version, M("1.3"))
         self.assertEqual(depend.python_tag, "cp27")
         self.assertEqual(depend.abi_tag, "cp27m")
         self.assertEqual(depend.platform_tag, "win32")
+        self.assertEqual(depend.platform_abi, "msvc2008")
 
-    def test_unsupported_metadata_version(self):
-        # Given
-        s = """\
+    def test_format_1_4(self):
+        r_depend = """\
 metadata_version = "1.4"
+name= "numpy_debug"
+version = "1.9.2"
+build = 3
 
-name = "foo"
-version = "1.0"
-build = 1
-
-arch = "amd64"
-platform = "darwin"
+arch = 'x86'
+platform = 'win32'
 osdist = None
 
 python = "2.7"
-
-abi_tag = "none"
-platform_tag = "macosx_10_6_i386"
 python_tag = "cp27"
+abi_tag = "cp27m"
+platform_tag = "win32"
 
-packages = []
+platform_abi = "msvc2008"
+
+packages = [
+]
 """
+        depend = LegacySpecDepend.from_string(r_depend)
+
+        # Then
+        self.assertEqual(depend._metadata_version, M("1.4"))
+        self.assertEqual(depend.python_tag, "cp27")
+        self.assertEqual(depend.abi_tag, "cp27m")
+        self.assertEqual(depend.platform_tag, "win32")
+        self.assertEqual(depend.platform_abi, "msvc2008")
+
+    def test_unsupported_metadata_version(self):
+        # Given
+        unsupported = MetadataVersion(
+            EggMetadata.HIGHEST_SUPPORTED_METADATA_VERSION.major + 1, 0
+        )
+
+        s = textwrap.dedent("""\
+            metadata_version = "{0}"
+
+            name = "foo"
+            version = "1.0"
+            build = 1
+
+            arch = "amd64"
+            platform = "darwin"
+            osdist = None
+
+            python = "2.7"
+
+            abi_tag = "none"
+            platform_tag = "macosx_10_6_i386"
+            python_tag = "cp27"
+
+            packages = []""".format(str(unsupported)))
 
         # When/Then
         with self.assertRaises(UnsupportedMetadata):
             LegacySpecDepend.from_string(s)
 
+    def test_error_python_to_python_tag(self):
+        # Given
+        s = textwrap.dedent("""\
+            metadata_version = "1.1"
+
+            name = "foo"
+            version = "1.0"
+            build = 1
+
+            arch = "amd64"
+            platform = "darwin"
+            osdist = None
+
+            python = "a.7"
+
+            packages = []""")
+
+        # When/Then
+        with self.assertRaisesRegexp(
+            InvalidMetadataField,
+            r"^Invalid value for metadata field 'python': u?'a.7'"
+        ) as exc:
+            LegacySpecDepend.from_string(s)
+
+        self.assertEqual(exc.exception.name, "python")
+        self.assertEqual(exc.exception.value, "a.7")
+
     def test_blacklisted_platform(self):
         # Given
         egg = XZ_5_2_0_EGG
@@ -264,6 +346,16 @@ packages = []
         # Then
         self.assertEqual(str(spec_depend._epd_legacy_platform), "win-32")
 
+    def test_missing_spec_depend(self):
+        # When/Then
+        with tempdir() as d:
+            path = os.path.join(d, 'egg-5.1-1.egg')
+            with zipfile2.ZipFile(path, "w") as zp:
+                zp.writestr("dummy", b"")
+
+            with self.assertRaises(MissingMetadata):
+                LegacySpecDepend.from_egg(path)
+
 
 class TestLegacySpecDependAbi(unittest.TestCase):
     def test_default_no_python_egg(self):
@@ -286,6 +378,7 @@ packages = []
 
         # Then
         self.assertIsNone(spec_depend.abi_tag, None)
+        self.assertEqual(spec_depend.platform_abi, "darwin")
 
     def test_default_pure_python_egg(self):
         # Given
@@ -307,6 +400,7 @@ packages = []
 
         # Then
         self.assertEqual(spec_depend.abi_tag, "cp27m")
+        self.assertEqual(spec_depend.platform_abi, "darwin")
 
     def test_default_extension_python_egg(self):
         # Given
@@ -330,6 +424,30 @@ packages = [
 
         # Then
         self.assertEqual(spec_depend.abi_tag, "cp27m")
+        self.assertEqual(spec_depend.platform_abi, "darwin")
+
+    def test_default_no_python_egg_windows(self):
+        # Given
+        spec_depend_string = """\
+metadata_version = '1.1'
+name = 'MKL'
+version = '11.1.4'
+build = 1
+
+arch = 'amd64'
+platform = 'win32'
+osdist = 'Windows'
+python = None
+packages = [
+]
+"""
+
+        # When
+        spec_depend = LegacySpecDepend.from_string(spec_depend_string)
+
+        # Then
+        self.assertIsNone(spec_depend.abi_tag)
+        self.assertEqual(spec_depend.platform_abi, "msvc2008")
 
     def test_default_pure_python_egg_pypi(self):
         # Given
@@ -353,6 +471,7 @@ packages = [
 
         # Then
         self.assertIsNone(spec_depend.abi_tag)
+        self.assertIsNone(spec_depend.platform_abi)
 
     def test_to_string(self):
         # Given
@@ -391,7 +510,7 @@ packages = []
         spec_depend = LegacySpecDepend.from_string(spec_depend_string)
         # A bit of an hack, but this is the only way to force a specific
         # metadata version for to_string for now
-        spec_depend._metadata_version = "1.3"
+        spec_depend._metadata_version = M("1.3")
         spec_depend_string = spec_depend.to_string()
 
         # Then
@@ -533,7 +652,9 @@ class TestParseRawspec(unittest.TestCase):
         spec_string = "metadata_version = '1.0'"
 
         # When/Then
-        with self.assertRaises(UnsupportedMetadata):
+        with self.assertRaisesRegexp(
+            InvalidMetadataField, r"^Missing metadata field 'name'"
+        ):
             parse_rawspec(spec_string)
 
     def test_simple_1_2(self):
@@ -666,7 +787,10 @@ packages = [
 """
 
         # When/Then
-        with self.assertRaises(UnsupportedMetadata):
+        with self.assertRaisesRegexp(
+            InvalidMetadataField,
+            r"^Invalid value for metadata field 'metadata_version': None"
+        ):
             parse_rawspec(spec_s)
 
         # Given a spec_string without some other metadata in >= 1.1
@@ -686,7 +810,9 @@ packages = [
 """
 
         # When/Then
-        with self.assertRaises(InvalidMetadata):
+        with self.assertRaisesRegexp(
+            InvalidMetadataField, r"^Missing metadata field 'platform'"
+        ):
             parse_rawspec(spec_s)
 
         # Given a spec_string without some other metadata in >= 1.2
@@ -707,11 +833,19 @@ packages = [
 """
 
         # When/Then
-        with self.assertRaises(InvalidMetadata):
+        with self.assertRaisesRegexp(
+            InvalidMetadataField, r"^Missing metadata field 'python_tag'"
+        ):
             parse_rawspec(spec_s)
 
 
-class TestEggInfo(unittest.TestCase):
+class TestEggMetadata(unittest.TestCase):
+    def setUp(self):
+        self.tempdir = tempfile.mkdtemp()
+
+    def tearDown(self):
+        shutil.rmtree(self.tempdir)
+
     def test_simple(self):
         # Given
         egg = ENSTALLER_EGG
@@ -728,13 +862,19 @@ class TestEggInfo(unittest.TestCase):
 
         # Then
         self.assertEqual(metadata.name, "enstaller")
-        self.assertEqual(metadata.metadata_version_info, (1, 1))
+        self.assertEqual(
+            metadata.metadata_version,
+            MetadataVersion.from_string("1.1")
+        )
+        self.assertEqual(metadata.metadata_version, M("1.1"))
         self.assertEqual(metadata.abi_tag, None)
         self.assertEqual(metadata.abi_tag_string, 'none')
         self.assertEqual(metadata.platform_tag, None)
         self.assertEqual(metadata.platform_tag_string, 'any')
         self.assertEqual(metadata.python_tag, None)
         self.assertEqual(metadata.python_tag_string, 'none')
+        self.assertEqual(metadata.platform_abi_tag, None)
+        self.assertEqual(metadata.platform_abi_tag_string, 'none')
 
     def test_simple_non_python_egg(self):
         # Given
@@ -747,15 +887,201 @@ class TestEggInfo(unittest.TestCase):
         self.assertEqual(metadata.egg_basename, "MKL")
         self.assertEqual(metadata.name, "mkl")
 
-        self.assertEqual(metadata.metadata_version_info, (1, 1))
+        self.assertEqual(metadata.metadata_version, M("1.1"))
         self.assertEqual(metadata.abi_tag, None)
         self.assertEqual(metadata.abi_tag_string, 'none')
         self.assertEqual(metadata.platform_tag, 'macosx_10_6_x86_64')
         self.assertEqual(metadata.platform_tag_string, 'macosx_10_6_x86_64')
+        self.assertEqual(metadata.platform_abi, PlatformABI(u'darwin'))
+        self.assertEqual(metadata.platform_abi_tag, u'darwin')
+        self.assertEqual(metadata.platform_abi_tag_string, u'darwin')
         self.assertEqual(metadata.python_tag, None)
         self.assertEqual(metadata.python_tag_string, 'none')
         self.assertEqual(metadata.runtime_dependencies, tuple())
 
+    def _override_spec_depend(self, egg, spec_depend_string):
+        new_egg = os.path.join(self.tempdir, os.path.basename(egg))
+        with zipfile2.ZipFile(egg) as zi:
+            to_copy = set(zi.namelist()) - set(("EGG-INFO/spec/depend",))
+            with zipfile2.ZipFile(new_egg, "w") as zo:
+                for archive in to_copy:
+                    fp = zi.open(archive)
+                    try:
+                        zo.writestr(archive, fp.read())
+                    finally:
+                        fp.close()
+
+                zo.writestr("EGG-INFO/spec/depend", spec_depend_string)
+
+        return new_egg
+
+    def test_platform_abi(self):
+        # Given
+        spec_depend = textwrap.dedent("""\
+            metadata_version = '1.4'
+            name = 'Qt'
+            version = '4.8.7'
+            build = 4
+
+            arch = 'amd64'
+            platform = 'win32'
+            osdist = None
+            python = None
+
+            python_tag = None
+            abi_tag = None
+            platform_tag = 'win_amd64'
+
+            platform_abi = 'msvc2010'
+
+            packages = []""")
+
+        r_spec_depend = textwrap.dedent("""\
+            metadata_version = '1.4'
+            name = 'Qt'
+            version = '4.8.7'
+            build = 4
+
+            arch = 'amd64'
+            platform = 'win32'
+            osdist = None
+            python = None
+
+            python_tag = None
+            abi_tag = None
+            platform_tag = 'win_amd64'
+
+            platform_abi = 'msvc2015'
+
+            packages = []
+        """)
+
+        egg = self._override_spec_depend(ENSTALLER_EGG, spec_depend)
+
+        # When
+        metadata = EggMetadata.from_egg(egg)
+
+        # Then
+        self.assertEqual(metadata.name, "qt")
+        self.assertEqual(metadata.metadata_version, M("1.4"))
+        self.assertIs(metadata.is_strictly_supported, True)
+        self.assertEqual(metadata.platform_abi_tag, u"msvc2010")
+        self.assertEqual(metadata.platform_abi_tag_string, u"msvc2010")
+
+        # When
+        metadata.platform_abi = None
+
+        # Then
+        self.assertIs(metadata.platform_abi, None)
+        self.assertEqual(metadata.platform_abi_tag_string, "none")
+
+        # When
+        metadata.platform_abi = PlatformABI(u"msvc2015")
+
+        # Then
+        self.assertMultiLineEqual(
+            metadata._spec_depend.to_string(), r_spec_depend
+        )
+
+    def test_platform_abi_no_python(self):
+        # Given
+        spec_depend = textwrap.dedent("""\
+            metadata_version = '1.3'
+
+            name = 'PythonDoc'
+            version = '2.7.3'
+            build = 1
+
+            arch = 'amd64'
+            platform = 'win32'
+            osdist = None
+            python = None
+
+            platform_tag = 'win_amd64'
+            abi_tag = 'cp27m'
+            python_tag = 'py27'
+
+            packages = [
+              'appinst',
+            ]
+        """)
+
+        egg = self._override_spec_depend(ENSTALLER_EGG, spec_depend)
+
+        # When
+        metadata = EggMetadata.from_egg(egg)
+
+        # Then
+        self.assertEqual(metadata.name, "pythondoc")
+        self.assertEqual(metadata.metadata_version, M("1.3"))
+        self.assertIs(metadata.is_strictly_supported, True)
+        self.assertIsNone(metadata.platform_abi)
+        self.assertEqual(metadata.platform_abi_tag_string, "none")
+
+    def test_support_higher_compatible_version(self):
+        # Given
+        spec_depend = textwrap.dedent("""\
+            metadata_version = '1.5'
+            name = 'enstaller'
+            version = '4.5.0'
+            build = 1
+
+            arch = None
+            platform = None
+            osdist = None
+            python = None
+
+            python_tag = None
+            abi_tag = None
+            platform_tag = None
+
+            platform_abi = None
+
+            packages = []""")
+
+        egg = self._override_spec_depend(ENSTALLER_EGG, spec_depend)
+
+        # When
+        metadata = EggMetadata.from_egg(egg)
+
+        # Then
+        self.assertEqual(metadata.name, "enstaller")
+        self.assertEqual(metadata.metadata_version, M("1.5"))
+        self.assertIs(metadata.is_strictly_supported, False)
+
+        # When/Then
+        with self.assertRaises(UnsupportedMetadata):
+            metadata._spec_depend
+
+    def test_support_lower_compatible_version(self):
+        # Given
+        spec_depend = textwrap.dedent("""\
+            metadata_version = '1.1'
+            name = 'nose'
+            version = '1.3.4'
+            build = 1
+
+            arch = 'amd64'
+            platform = 'linux2'
+            osdist = 'RedHat_6'
+            python = '2.7'
+            packages = []\n""")
+
+        egg = NOSE_1_3_4_RH6_X86_64
+        metadata = EggMetadata.from_egg(egg)
+
+        # When
+        metadata = EggMetadata.from_egg_metadata(
+            metadata, metadata_version=M("1.1")
+        )
+
+        # Then
+        self.assertEqual(metadata.name, "nose")
+        self.assertEqual(metadata.metadata_version, M("1.1"))
+        self.assertMultiLineEqual(
+            metadata._spec_depend.to_string(), spec_depend
+        )
+
     def test_from_cross_platform_egg(self):
         # Given
         egg = ENSTALLER_EGG
@@ -771,29 +1097,29 @@ class TestEggInfo(unittest.TestCase):
         self.assertEqual(metadata.build, 1)
         self.assertEqual(metadata.upstream_version, "4.5.0")
         self.assertIsNone(metadata.python_tag)
-        self.assertEqual(metadata.metadata_version_info, (1, 1))
+        self.assertEqual(metadata.metadata_version, M("1.1"))
 
     def test_from_platform_egg(self):
         # Given
         egg = ETS_EGG
         r_runtime_dependencies = (
-            Requirement.from_spec_string('apptools 4.2.0-2'),
-            Requirement.from_spec_string('blockcanvas 4.0.3-1'),
-            Requirement.from_spec_string('casuarius 1.1-1'),
-            Requirement.from_spec_string('chaco 4.3.0-2'),
-            Requirement.from_spec_string('codetools 4.1.0-2'),
-            Requirement.from_spec_string('enable 4.3.0-5'),
-            Requirement.from_spec_string('enaml 0.6.8-2'),
-            Requirement.from_spec_string('encore 0.3-1'),
-            Requirement.from_spec_string('envisage 4.3.0-2'),
-            Requirement.from_spec_string('etsdevtools 4.0.2-1'),
-            Requirement.from_spec_string('etsproxy 0.1.2-1'),
-            Requirement.from_spec_string('graphcanvas 4.0.2-1'),
-            Requirement.from_spec_string('mayavi 4.3.0-3'),
-            Requirement.from_spec_string('pyface 4.3.0-2'),
-            Requirement.from_spec_string('scimath 4.1.2-2'),
-            Requirement.from_spec_string('traits 4.3.0-2'),
-            Requirement.from_spec_string('traitsui 4.3.0-2'),
+            Requirement.from_spec_string(u'apptools 4.2.0-2'),
+            Requirement.from_spec_string(u'blockcanvas 4.0.3-1'),
+            Requirement.from_spec_string(u'casuarius 1.1-1'),
+            Requirement.from_spec_string(u'chaco 4.3.0-2'),
+            Requirement.from_spec_string(u'codetools 4.1.0-2'),
+            Requirement.from_spec_string(u'enable 4.3.0-5'),
+            Requirement.from_spec_string(u'enaml 0.6.8-2'),
+            Requirement.from_spec_string(u'encore 0.3-1'),
+            Requirement.from_spec_string(u'envisage 4.3.0-2'),
+            Requirement.from_spec_string(u'etsdevtools 4.0.2-1'),
+            Requirement.from_spec_string(u'etsproxy 0.1.2-1'),
+            Requirement.from_spec_string(u'graphcanvas 4.0.2-1'),
+            Requirement.from_spec_string(u'mayavi 4.3.0-3'),
+            Requirement.from_spec_string(u'pyface 4.3.0-2'),
+            Requirement.from_spec_string(u'scimath 4.1.2-2'),
+            Requirement.from_spec_string(u'traits 4.3.0-2'),
+            Requirement.from_spec_string(u'traitsui 4.3.0-2'),
         )
 
         # When
@@ -808,7 +1134,7 @@ class TestEggInfo(unittest.TestCase):
         )
         self.assertEqual(metadata.build, 3)
         self.assertEqual(metadata.upstream_version, "4.3.0")
-        self.assertEqual(metadata.metadata_version_info, (1, 1))
+        self.assertEqual(metadata.metadata_version, M("1.1"))
         self.assertEqual(
             metadata.platform, EPDPlatform.from_epd_string("rh5-32")
         )
@@ -902,7 +1228,8 @@ class TestEggInfo(unittest.TestCase):
             metadata = EggMetadata.from_egg(egg)
 
         # Then
-        self.assertEqual(metadata.python_tag, "py27")
+        self.assertEqual(metadata.python_tag, "cp27")
+        self.assertEqual(metadata.abi_tag, "cp27m")
 
         # When
         with mock.patch(
@@ -913,7 +1240,7 @@ class TestEggInfo(unittest.TestCase):
                 metadata = EggMetadata.from_egg(zp)
 
         # Then
-        self.assertEqual(metadata.python_tag, "py27")
+        self.assertEqual(metadata.python_tag, "cp27")
 
     def test_blacklisted_platform(self):
         # Given
@@ -956,9 +1283,9 @@ class TestEggInfo(unittest.TestCase):
 
     def test_blacklisted_pkg_info(self):
         # Given
-        egg = FAKE_PYSIDE_1_1_0_EGG
-        mock_sha256 = ("5eff70cfb464c2d531e6f93f7601e8ef8255b3a1ab4"
-                       "dd533826cfdcd5b962b60")
+        egg = FAKE_MEDIALOG_BOARDFILE_1_6_1_EGG
+        mock_sha256 = ("ab9e029caf273e4a251d3686425cd4225e1b97682749acc7a82d"
+                       "c1fd15dbd060")
 
         # When
         with mock.patch(
@@ -968,11 +1295,12 @@ class TestEggInfo(unittest.TestCase):
             metadata = EggMetadata.from_egg(egg)
 
         # Then
+        self.maxDiff = None
         pkg_info = metadata.pkg_info
         self.assertEqual(pkg_info.metadata_version, "1.0")
-        self.assertEqual(pkg_info.name, "PySide")
+        self.assertEqual(pkg_info.name, "medialog.boardfile")
         self.assertMultiLineEqual(
-            pkg_info.description, FAKE_PYSIDE_1_1_0_EGG_PKG_INFO
+            pkg_info.description, FAKE_MEDIALOG_BOARDFILE_1_6_1_PKG_INFO
         )
 
         # When
@@ -986,9 +1314,9 @@ class TestEggInfo(unittest.TestCase):
         # Then
         pkg_info = metadata.pkg_info
         self.assertEqual(pkg_info.metadata_version, "1.0")
-        self.assertEqual(pkg_info.name, "PySide")
+        self.assertEqual(pkg_info.name, "medialog.boardfile")
         self.assertMultiLineEqual(
-            pkg_info.description, FAKE_PYSIDE_1_1_0_EGG_PKG_INFO
+            pkg_info.description, FAKE_MEDIALOG_BOARDFILE_1_6_1_PKG_INFO
         )
 
         # Given
@@ -1033,3 +1361,132 @@ class TestEggInfo(unittest.TestCase):
             metadata.pkg_info.author_email,
             u"johannes.buchner.acad [\ufffdt] gmx.com",
         )
+
+    def test_dump_simple(self):
+        # Given
+        egg = ENSTALLER_EGG
+        r_metadata = EggMetadata.from_egg(egg)
+
+        path = os.path.join(self.tempdir, "foo.zip")
+
+        # When
+        r_metadata.dump(path)
+
+        # Then
+        metadata = EggMetadata.from_egg(path)
+        self.assertEqual(metadata, r_metadata)
+
+    def test_dump_blacklisted(self):
+        self.maxDiff = None
+
+        # Given
+        egg = FAKE_MEDIALOG_BOARDFILE_1_6_1_EGG
+        mock_sha256 = (
+            "ab9e029caf273e4a251d3686425cd4225e1b97682749acc7a82dc1fd15dbd060"
+        )
+
+        with mock.patch(
+            "okonomiyaki.file_formats._egg_info.compute_sha256",
+            return_value=mock_sha256
+        ):
+            r_metadata = EggMetadata.from_egg(egg)
+
+        path = os.path.join(self.tempdir, "foo.zip")
+
+        # When
+        r_metadata.dump(path)
+
+        # Then
+        metadata = EggMetadata.from_egg(path)
+        self.assertMultiLineEqual(
+            metadata.pkg_info.description,
+            FAKE_MEDIALOG_BOARDFILE_1_6_1_PKG_INFO
+        )
+        self.assertEqual(metadata, r_metadata)
+
+    def test_dump_blacklisted_platform(self):
+        # Given
+        egg = XZ_5_2_0_EGG
+        mock_sha256 = (
+            "ca5f2c417dd9f6354db3c2999edb441382ed11c7a034ade1839d1871a78ab2e8"
+        )
+
+        with mock.patch(
+            "okonomiyaki.file_formats._egg_info.compute_sha256",
+            return_value=mock_sha256
+        ):
+            r_metadata = EggMetadata.from_egg(egg)
+
+        path = os.path.join(self.tempdir, "foo.zip")
+
+        # When
+        r_metadata.dump(path)
+
+        # Then
+        metadata = EggMetadata.from_egg(path)
+        self.assertEqual(metadata.platform_tag, "win32")
+
+    def test_to_json_dict(self):
+        # Given
+        egg = NOSE_1_3_4_RH5_X86_64
+        metadata = EggMetadata.from_egg(egg)
+
+        r_json_dict = {
+            "metadata_version": u"1.3",
+            "_raw_name": u"nose",
+            "version": u"1.3.4-1",
+            "epd_platform": u"rh5_x86_64",
+            "python_tag": u"cp27",
+            "abi_tag": u"cp27m",
+            "platform_tag": u"linux_x86_64",
+            "platform_abi_tag": u"gnu",
+            "runtime_dependencies": [],
+            "summary": (
+                u"Extends the Python Unittest module with additional "
+                "disocvery and running\noptions\n"
+            )
+        }
+
+        # When
+        json_dict = metadata.to_json_dict()
+
+        # Then
+        self.assertEqual(json_dict, r_json_dict)
+
+    def test_from_json_dict(self):
+        # Given
+        egg = NOSE_1_3_4_RH5_X86_64
+        r_metadata = EggMetadata.from_egg(egg)
+
+        json_dict = {
+            "metadata_version": u"1.3",
+            "_raw_name": u"nose",
+            "version": u"1.3.4-1",
+            "epd_platform": u"rh5_x86_64",
+            "python_tag": u"cp27",
+            "abi_tag": u"cp27m",
+            "platform_tag": u"linux_x86_64",
+            "platform_abi_tag": u"gnu",
+            "runtime_dependencies": [],
+            "summary": (
+                u"Extends the Python Unittest module with additional "
+                "disocvery and running\noptions\n"
+            )
+        }
+
+        # When
+        metadata = EggMetadata.from_json_dict(json_dict, r_metadata.pkg_info)
+
+        # Then
+        self.assertEqual(metadata, r_metadata)
+
+    def _test_roundtrip(self, egg):
+        r_metadata = EggMetadata.from_egg(egg)
+        metadata = EggMetadata.from_json_dict(
+            r_metadata.to_json_dict(), r_metadata.pkg_info
+        )
+
+        self.assertEqual(metadata, r_metadata)
+
+    def test_mkl_roundtrip(self):
+        self._test_roundtrip(MKL_10_3_RH5_X86_64)
