@@ -8,9 +8,10 @@ import six
 from attr import attributes, attr
 from attr.validators import instance_of
 
-from ..errors import OkonomiyakiError
+from ..versions import RuntimeVersion
+from ..errors import OkonomiyakiError, InvalidPEP440Version
 from ._arch import Arch, ArchitectureKind
-from .platform import OSKind, FamilyKind, NameKind, Platform
+from ._platform import OSKind, FamilyKind, NameKind, Platform
 
 # the string used in EGG-INFO/spec/depend. Only used during normalization
 # operations.
@@ -28,7 +29,7 @@ _ARCHBITS_TO_ARCH = {
 X86 = Arch(ArchitectureKind.x86)
 X86_64 = Arch(ArchitectureKind.x86_64)
 
-PLATFORM_NAMES = [
+PLATFORM_NAMES = (
     "osx",
     "rh3",
     "rh5",
@@ -36,9 +37,9 @@ PLATFORM_NAMES = [
     "rh7",
     "sol",
     "win",
-]
+)
 
-EPD_PLATFORM_SHORT_NAMES = [
+EPD_PLATFORM_SHORT_NAMES = (
     "osx-32",
     "osx-64",
     "rh3-32",
@@ -51,17 +52,21 @@ EPD_PLATFORM_SHORT_NAMES = [
     "sol-64",
     "win-32",
     "win-64",
-]
+)
 
-_EPD_PLATFORM_STRING_RE = re.compile("""
+VALID_PLATFORMS_FILTER = PLATFORM_NAMES + ("all", "rh",)
+
+_EPD_PLATFORM_STRING_RE = re.compile(r"""
+    ^
     (?P<os>[^-_]+)
     [_-]
     (?P<arch>[^-]+)
+    $
     """, flags=re.VERBOSE)
 
-_LINUX_TAG_R = re.compile("^linux_(?P<arch>\S+)$")
-_MACOSX_TAG_R = re.compile("^macosx_([^_]+)_([^_]+)_(?P<arch>\S+)$")
-_WINDOWS_TAG_R = re.compile("^win_*(?P<arch>\S+)$")
+_LINUX_TAG_R = re.compile(r"^linux_(?P<arch>\S+)$")
+_MACOSX_TAG_R = re.compile(r"^macosx_([^_]+)_([^_]+)_(?P<arch>\S+)$")
+_WINDOWS_TAG_R = re.compile(r"^win_*(?P<arch>\S+)$")
 
 _ANY_PLATFORM_STRING = u'any'
 
@@ -77,7 +82,7 @@ def platform_validator():
 
 
 @six.python_2_unicode_compatible
-@attributes
+@attributes(frozen=True)
 class EPDPlatform(object):
     """
     An sane Canopy/EPD platform representation.
@@ -129,12 +134,18 @@ class EPDPlatform(object):
         return _guess_epd_platform(arch)
 
     @classmethod
-    def from_string(cls, s):
-        """
-        Create a new instance from an epd platform string (e.g. 'win-32')
+    def from_string(cls, s, runtime_version=None):
+        """ Create a new instance from an epd platform string.
 
-        Note: new, more explicit architecture names are also supported, e.g.
-        'win-x86' or 'win-x86_64' are supported.
+        Parameters:
+        s : string
+           The platform string e.g. win-32. New, more explicit
+           architecture names are also supported, e.g. 'win-x86' or
+           'win-x86_64' are supported.
+        runtime_version: RuntimeVersion
+           Passing the runtime version will result in more precise Platform
+           description taking into account the historical records of the
+           Enthought python runtime releases.
         """
         m = _EPD_PLATFORM_STRING_RE.match(s)
         if m is None:
@@ -148,12 +159,13 @@ class EPDPlatform(object):
             else:
                 arch_name = _ARCHBITS_TO_ARCH[arch_bits]
                 arch = machine = Arch.from_name(arch_name)
-            os, name, family, release = _epd_name_to_quadruplet(platform_name)
+            os, name, family, release = _epd_name_and_python_to_quadruplet(
+                platform_name, runtime_version)
             platform = Platform(os, name, family, release, arch, machine)
             return cls(platform)
 
     @classmethod
-    def from_epd_string(cls, s):
+    def from_epd_string(cls, s, runtime_version=None):
         """
         Create a new instance from an epd platform string (e.g. 'win-32')
 
@@ -163,13 +175,14 @@ class EPDPlatform(object):
             "Deprecated: use EPDPlatform.from_string instead",
             DeprecationWarning
         )
-        return cls.from_string(s)
+        return cls.from_string(s, runtime_version)
 
     @classmethod
-    def _from_spec_depend_data(cls, platform, osdist, arch_name):
+    def _from_spec_depend_data(
+            cls, platform, osdist, arch_name,
+            platform_tag, python_version, platform_abi):
         msg = ("Unrecognized platform/osdist combination: {0!r}/{1!r}"
                .format(platform, osdist))
-
         arch = Arch.from_name(arch_name)
         if platform == "darwin":
             epd_name = "osx"
@@ -188,7 +201,23 @@ class EPDPlatform(object):
                 raise ValueError(msg)
         else:
             raise ValueError(msg)
-        return cls.from_epd_string(u"{0}-{1}".format(epd_name, arch._arch_bits))
+
+        if python_version is not None:
+            try:
+                python_version = RuntimeVersion.from_string(python_version)
+            except InvalidPEP440Version:
+                python_version = None
+
+        os_kind, name_kind, family_kind, release = _epd_name_and_python_to_quadruplet(
+            epd_name, python_version)
+
+        if 'osx' in platform_tag.lower():
+            release = '.'.join(platform_tag.split('_')[1:3])
+
+        platform = Platform(
+            os_kind=os_kind, name_kind=name_kind, family_kind=family_kind,
+            release=release, arch=arch, machine=arch)
+        return cls(platform)
 
     @classmethod
     def _from_platform_tag(cls, platform_tag):
@@ -240,29 +269,31 @@ class EPDPlatform(object):
     def pep425_tag(self):
         msg = "Cannot guess platform tag for platform {0!r}"
 
-        if self.platform.os_kind == OSKind.darwin:
-            if self.platform.arch == X86:
-                return u"macosx_10_6_i386"
-            elif self.platform.arch == X86_64:
-                return u"macosx_10_6_x86_64"
+        platform = self.platform
+        if platform.os_kind == OSKind.darwin:
+            release = platform.release.replace('.', '_')
+            if platform.arch == X86:
+                return u"macosx_{}_i386".format(release)
+            elif platform.arch == X86_64:
+                return u"macosx_{}_x86_64".format(release)
             else:
-                raise OkonomiyakiError(msg.format(self.platform))
-        elif self.platform.os_kind == OSKind.linux:
-            if self.platform.arch == X86:
+                raise OkonomiyakiError(msg.format(platform))
+        elif platform.os_kind == OSKind.linux:
+            if platform.arch == X86:
                 return u"linux_i686"
-            elif self.platform.arch == X86_64:
+            elif platform.arch == X86_64:
                 return u"linux_x86_64"
             else:
-                raise OkonomiyakiError(msg.format(self.platform))
-        elif self.platform.os_kind == OSKind.windows:
-            if self.platform.arch == X86:
+                raise OkonomiyakiError(msg.format(platform))
+        elif platform.os_kind == OSKind.windows:
+            if platform.arch == X86:
                 return u"win32"
-            elif self.platform.arch == X86_64:
+            elif platform.arch == X86_64:
                 return u"win_amd64"
             else:
-                raise OkonomiyakiError(msg.format(self.platform))
+                raise OkonomiyakiError(msg.format(platform))
         else:
-            raise OkonomiyakiError(msg.format(self.platform))
+            raise OkonomiyakiError(msg.format(platform))
 
     @property
     def platform_name(self):
@@ -326,11 +357,27 @@ def applies(platform_string, to='current'):
 
         parts = component.split("-")
         if len(parts) == 1:
-            return parts[0], None
+            if parts[0] in VALID_PLATFORMS_FILTER:
+                return parts[0], None
+            elif parts[0] in _ARCHBITS_TO_ARCH:
+                return "all", parts[0]
+            else:
+                raise ValueError(
+                    "Invalid filter string: '{}'".format(component)
+                )
         elif len(parts) == 2:
+            if (
+                parts[0] not in VALID_PLATFORMS_FILTER
+                or parts[1] not in _ARCHBITS_TO_ARCH
+            ):
+                raise ValueError(
+                    "Invalid filter string: '{}'".format(component)
+                )
             return parts[0], parts[1]
         else:
-            raise ValueError()
+            raise ValueError(
+                "Invalid filter string: '{}'".format(component)
+            )
 
     def _are_compatible(short_left, short_right):
         return short_left == short_right or \
@@ -382,7 +429,8 @@ def applies(platform_string, to='current'):
         return any(conditions)
 
 
-def _epd_name_to_quadruplet(name):
+def _epd_name_and_python_to_quadruplet(name, runtime_version=None):
+    py38 = RuntimeVersion.from_string('3.8')
     if name == "rh7":
         return (OSKind.linux, NameKind.rhel, FamilyKind.rhel, "7.1")
     if name == "rh6":
@@ -392,11 +440,17 @@ def _epd_name_to_quadruplet(name):
     elif name == "rh3":
         return (OSKind.linux, NameKind.rhel, FamilyKind.rhel, "3.8")
     elif name == "osx":
-        return (OSKind.darwin, NameKind.mac_os_x, FamilyKind.mac_os_x, "10.6")
+        if runtime_version is not None and runtime_version >= py38:
+            return (OSKind.darwin, NameKind.mac_os_x, FamilyKind.mac_os_x, "10.14")
+        else:
+            return (OSKind.darwin, NameKind.mac_os_x, FamilyKind.mac_os_x, "10.6")
     elif name == "sol":
         return (OSKind.solaris, NameKind.solaris, FamilyKind.solaris, "")
     elif name == "win":
-        return (OSKind.windows, NameKind.windows, FamilyKind.windows, "")
+        if runtime_version is not None and runtime_version >= py38:
+            return (OSKind.windows, NameKind.windows, FamilyKind.windows, "10")
+        else:
+            return (OSKind.windows, NameKind.windows, FamilyKind.windows, "")
     else:
         msg = "Invalid epd platform string name: {0!r}".format(name)
         raise OkonomiyakiError(msg)
